@@ -47,6 +47,8 @@
 
 #define SPACE 16
 #define XND_N_MONITORS xfce_notify_daemon_get_n_monitors_quark()
+#define KNOWN_APPLICATIONS_PROP       "/applications/known_applications"
+#define MUTED_APPLICATIONS_PROP       "/applications/muted_applications"
 
 struct _XfceNotifyDaemon
 {
@@ -58,6 +60,8 @@ struct _XfceNotifyDaemon
     gdouble initial_opacity;
     GtkCornerType notify_location;
     gboolean do_fadeout;
+    gboolean primary_monitor;
+    gboolean do_not_disturb;
 
     GtkCssProvider *css_provider;
     gboolean is_default_theme;
@@ -120,6 +124,11 @@ static gboolean notify_get_capabilities (XfceNotifyGBus *skeleton,
                                 		 GDBusMethodInvocation   *invocation,
                                          XfceNotifyDaemon *xndaemon);
 
+static void notify_update_known_applications (XfconfChannel *channel,
+                                              gchar *app_name);
+
+static gboolean notify_application_is_muted (XfconfChannel *channel,
+                                             gchar *new_app_name);
 
 static gboolean notify_notify (XfceNotifyGBus *skeleton,
                                GDBusMethodInvocation   *invocation,
@@ -663,10 +672,10 @@ xfce_notify_daemon_window_size_allocate(GtkWidget *widget,
     XfceNotifyWindow *window = XFCE_NOTIFY_WINDOW(widget);
     GdkScreen *p_screen = NULL;
     GdkDevice *pointer;
+    GdkScreen *widget_screen;
 #if GTK_CHECK_VERSION (3, 20, 0)
     GdkSeat *seat;
 #else
-    GdkScreen *widget_screen;
     GdkDisplay *display;
     GdkDeviceManager *device_manager;
 #endif
@@ -699,11 +708,11 @@ xfce_notify_daemon_window_size_allocate(GtkWidget *widget,
         xndaemon->reserved_rectangles[screen_n][monitor] = old_list;
     }
 
+    widget_screen = gtk_widget_get_screen (widget);
 #if GTK_CHECK_VERSION (3, 20, 0)
     seat = gdk_display_get_default_seat (gdk_display_get_default());
     pointer = gdk_seat_get_pointer (seat);
 #else
-    widget_screen = gtk_widget_get_screen (widget);
     display = gdk_screen_get_display (widget_screen);
     device_manager = gdk_display_get_device_manager (display);
     pointer = gdk_device_manager_get_client_pointer (device_manager);
@@ -711,7 +720,10 @@ xfce_notify_daemon_window_size_allocate(GtkWidget *widget,
 
     gdk_device_get_position (pointer, &p_screen, &x, &y);
 
-    monitor = gdk_screen_get_monitor_at_point(p_screen, x, y);
+    if (xndaemon->primary_monitor == TRUE)
+        monitor = gdk_screen_get_primary_monitor(widget_screen);
+    else
+        monitor = gdk_screen_get_monitor_at_point(p_screen, x, y);
     screen_n = gdk_screen_get_number (p_screen);
 
     DBG("We are on the monitor %i, screen %i", monitor, screen_n);
@@ -1029,23 +1041,98 @@ notify_update_theme_foreach (gpointer key, gpointer value, gpointer data)
     return FALSE;
 }
 
-static gboolean notify_notify (XfceNotifyGBus *skeleton,
-                               GDBusMethodInvocation   *invocation,
-                               const gchar *app_name,
-                               guint replaces_id,
-                               const gchar *app_icon,
-                               const gchar *summary,
-                               const gchar *body,
-                               const gchar **actions,
-                               GVariant *hints,
-                               gint expire_timeout,
-                               XfceNotifyDaemon *xndaemon)
+static void
+notify_update_known_applications (XfconfChannel *channel, gchar *new_app_name)
+{
+    GPtrArray *known_applications;
+    GValue *val;
+    gint index = 0;
+    gint index_before = 0;
+    gint index_after = 0;
+
+    val = g_new0 (GValue, 1);
+    g_value_init (val, G_TYPE_STRING);
+    g_value_take_string (val, new_app_name);
+
+    known_applications = xfconf_channel_get_arrayv (channel, KNOWN_APPLICATIONS_PROP);
+    /* No known applications, initialize the channel and property */
+    if (known_applications == NULL || known_applications->len < 1) {
+        GPtrArray *array;
+        array = g_ptr_array_sized_new (1);
+        g_ptr_array_add (array, val);
+        if (!xfconf_channel_set_arrayv (channel, KNOWN_APPLICATIONS_PROP, array))
+            g_warning ("Could not initialize the application log: %s", new_app_name);
+        xfconf_array_free (array);
+    }
+    /* Add the new application to the list unless it's already known */
+    else {
+        guint i;
+        gboolean application_is_known = FALSE;
+        /* Check if the application is actually unknown */
+        for (i = 0; i < known_applications->len; i++) {
+            GValue *known_application;
+            known_application = g_ptr_array_index (known_applications, i);
+            /* Remember where to put the application in alphabetical order */
+            if (g_ascii_strcasecmp (g_value_get_string (known_application), new_app_name) < 0)
+                index = i + 1;
+            /* Just to be sure that we've found the exact same application don't ignore the case when comparing strings here */
+            else if (g_strcmp0 (new_app_name, g_value_get_string (known_application)) == 0) {
+                application_is_known = TRUE;
+                break;
+            }
+        }
+        /* Unknown application, add it in alphabetical order */
+        if (application_is_known == FALSE) {
+            g_ptr_array_insert (known_applications, index, val);
+            if (!xfconf_channel_set_arrayv (channel, KNOWN_APPLICATIONS_PROP, known_applications))
+                g_warning ("Could not add a new application to the log: %s", new_app_name);
+        }
+    }
+    xfconf_array_free (known_applications);
+}
+
+static gboolean
+notify_application_is_muted (XfconfChannel *channel, gchar *new_app_name)
+{
+    GPtrArray *muted_applications;
+    guint i;
+
+    muted_applications = xfconf_channel_get_arrayv (channel, MUTED_APPLICATIONS_PROP);
+
+    /* Check whether this application should be muted */
+    if (muted_applications != NULL) {
+        for (i = 0; i < muted_applications->len; i++) {
+            GValue *muted_application;
+            muted_application = g_ptr_array_index (muted_applications, i);
+            if (g_str_match_string (new_app_name, g_value_get_string (muted_application), FALSE) == TRUE) {
+                return TRUE;
+            }
+        }
+    }
+
+    xfconf_array_free (muted_applications);
+    return FALSE;
+}
+
+static gboolean
+notify_notify (XfceNotifyGBus *skeleton,
+               GDBusMethodInvocation   *invocation,
+               const gchar *app_name,
+               guint replaces_id,
+               const gchar *app_icon,
+               const gchar *summary,
+               const gchar *body,
+               const gchar **actions,
+               GVariant *hints,
+               gint expire_timeout,
+               XfceNotifyDaemon *xndaemon)
 {
     XfceNotifyWindow *window;
     GdkPixbuf *pix = NULL;
     GVariant *image_data = NULL;
     const gchar *image_path = NULL;
     const gchar *desktop_id = NULL;
+    gchar *new_app_name;
     gint value_hint = 0;
     gboolean value_hint_set = FALSE;
     gboolean x_canonical = FALSE;
@@ -1106,9 +1193,36 @@ static gboolean notify_notify (XfceNotifyGBus *skeleton,
         g_variant_unref (item);
     }
 
+    if (desktop_id)
+        new_app_name = g_strdup (desktop_id);
+    else
+        new_app_name = g_strdup (app_name);
+
+    notify_update_known_applications (xndaemon->settings, new_app_name);
+
     if(expire_timeout == -1)
         expire_timeout = xndaemon->expire_timeout;
 
+    /* Don't show notification bubbles in the "Do not disturb" mode or if the
+       application has been muted by the user. Exceptions are "urgent"
+       notifications which do not expire. */
+    if (expire_timeout != 0)
+    {
+        if (xndaemon->do_not_disturb == TRUE ||
+            notify_application_is_muted (xndaemon->settings, new_app_name) == TRUE)
+        {
+            /* Reset the close timeout since we're processing a new notification,
+               even if we're not showing a notification bubble */
+            if(xndaemon->close_timeout)
+                g_source_remove(xndaemon->close_timeout);
+
+            xndaemon->close_timeout = 0;
+
+            xfce_notify_gbus_complete_notify(skeleton, invocation, OUT_id);
+            return TRUE;
+        }
+    }
+
     if(replaces_id
        && (window = g_tree_lookup(xndaemon->active_notifications,
                                   GUINT_TO_POINTER(replaces_id))))
@@ -1163,7 +1277,7 @@ static gboolean notify_notify (XfceNotifyGBus *skeleton,
     else if (image_path) {
         xfce_notify_window_set_icon_name (window, image_path);
     }
-    else if (app_icon) {
+    else if (app_icon && (g_strcmp0 (app_icon, "") != 0)) {
         xfce_notify_window_set_icon_name(window, app_icon);
     }
     else {
@@ -1181,7 +1295,7 @@ static gboolean notify_notify (XfceNotifyGBus *skeleton,
                     if(icon_file) {
                         pix = gtk_icon_theme_load_icon(gtk_icon_theme_get_default(),
                                                        icon_file,
-                                                       GTK_ICON_SIZE_DIALOG,
+                                                       48,
                                                        GTK_ICON_LOOKUP_FORCE_SIZE,
                                                        NULL);
 
@@ -1410,6 +1524,14 @@ xfce_notify_daemon_settings_changed(XfconfChannel *channel,
         xndaemon->do_fadeout = G_VALUE_TYPE(value)
                                ? g_value_get_boolean(value)
                                : TRUE;
+    } else if(!strcmp(property, "/primary-monitor")) {
+        xndaemon->primary_monitor = G_VALUE_TYPE(value)
+                                    ? g_value_get_boolean(value)
+                                    : TRUE;
+    } else if(!strcmp(property, "/do-not-disturb")) {
+        xndaemon->do_not_disturb = G_VALUE_TYPE(value)
+                                 ? g_value_get_boolean(value)
+                                 : FALSE;
     }
 }
 
@@ -1443,6 +1565,14 @@ xfce_notify_daemon_load_config (XfceNotifyDaemon *xndaemon,
 
     xndaemon->do_fadeout = xfconf_channel_get_bool(xndaemon->settings,
                                                 "/do-fadeout", TRUE);
+    xndaemon->primary_monitor = xfconf_channel_get_bool(xndaemon->settings,
+                                                        "/primary-monitor", FALSE);
+
+    xndaemon->do_not_disturb = xfconf_channel_get_bool(xndaemon->settings,
+                                                       "/do-not-disturb",
+                                                       FALSE);
+    /* Clean up old notifications from the backlog */
+    xfconf_channel_reset_property (xndaemon->settings, "/backlog", TRUE);
 
     g_signal_connect(G_OBJECT(xndaemon->settings), "property-changed",
                      G_CALLBACK(xfce_notify_daemon_settings_changed),
